#!/usr/bin/env python
"""./testsuite.py [options] [test name]

Run the templates_parser testsuite
"""

from gnatpython.env import Env
from gnatpython.ex import Run
from gnatpython.fileutils import mkdir, rm
from gnatpython.main import Main
from gnatpython.mainloop import (MainLoop, add_mainloop_options,
                                 generate_collect_result)
from gnatpython.testdriver import add_run_test_options
from gnatpython.reports import ReportDiff

from glob import glob

import os
import sys

#  Compute PATH to global test drivers (testme, print_tree)

TARGET = os.environ.get("TARGET")
PRJ_BUILD = os.environ.get("PRJ_BUILD")

if TARGET is None:
    TARGET = Run(['gcc', '-dumpmachine']).out.strip('\n')
else:
    TARGET = TARGET.lower()

if PRJ_BUILD is None:
    PRJ_BUILD = "debug"
else:
    PRJ_BUILD = PRJ_BUILD.lower()


def makedir(dir):
    return os.getcwd() + "/../.build/" + dir + "/" \
        + TARGET + "/" + PRJ_BUILD + "/static/"

os.environ["PATH"] = os.environ.get("PATH") + os.pathsep + makedir("bin") \
    + os.pathsep + makedir("rbin")

from gnatpython.ex import Run


def gprbuild(prj):
    """Compile a project with gprbuild"""
    cmd = ["gprbuild", "-p", "-gnat05", "-P" + prj, "-bargs", "-E"]
    process = Run(cmd)
    if process.status:
        print process.out


def main():
    """Run the testsuite"""
    options = __parse_options()
    assert os.path.exists(makedir("bin")), \
        "cannot find %s directory" % makedir("bin")
    assert os.path.exists(makedir("rbin")), \
        "cannot find %s directory" % makedir("rbin")
    env = Env()
    env.add_search_path("PYTHONPATH", os.getcwd())

    test_list = [t for t in filter_list('tests/*', options.run_test)
                 if os.path.isdir(t)]

    # Various files needed or created by the testsuite
    result_dir = options.output_dir
    results_file = result_dir + '/results'

    if os.path.exists(result_dir):
        rm(result_dir, True)

    mkdir(result_dir)

    discs = env.discriminants

    if options.discs:
        discs += options.discs

    def test_build_cmd(test, _):
        """Run the given test"""
        cmd = [sys.executable, 'run-test',
               '-d', ",".join(discs),
               '-o', result_dir,
               '-t', options.tmp,
               test]
        if options.verbose:
            cmd.append('-v')
        if options.host:
            cmd.append('--host=' + options.host)
        if options.target:
            cmd.append('--target=' + options.target)
        if not options.enable_cleanup:
            cmd.append('--disable-cleanup')
        return Run(cmd, bg=True)

    collect_result = generate_collect_result(
        result_dir, results_file, options.view_diffs)

    MainLoop(test_list, test_build_cmd, collect_result, options.mainloop_jobs)

    # Write report
    with open(result_dir + '/discs', 'w') as discs_f:
        discs_f.write(" ".join(discs))
    ReportDiff(result_dir, options.old_result_dir).txt_image(
        result_dir + '/report')


def filter_list(pattern, run_test=""):
    """Compute the list of test matching pattern

    If run_test is not null, run only tests containing run_test
    """
    test_list = glob(pattern)
    if not run_test:
        return test_list
    else:
        return [test for test in test_list if run_test in test]


def __parse_options():
    """Parse command lines options"""
    m = Main(add_targets_options=True)
    add_mainloop_options(m)
    add_run_test_options(m)
    m.add_option("--diffs", dest="view_diffs", action="store_true",
                 default=False, help="Print .diff content")
    m.add_option("--old-result-dir", type="string", default=None,
                 help="Old result dir")
    m.parse_args()

    if m.args:
        m.options.run_test = m.args[0]
        # User want to run only one test
        print "Running only test '%s'" % m.options.run_test
    else:
        m.options.run_test = ""

    if m.options.discs:
        m.options.discs = m.options.discs.split(',')

    return m.options

if __name__ == "__main__":
    try:
        main()
    except AssertionError, e:
        print 'ERROR: %s' % e



from test_support import *


def print_file(content):
    for line in content:
        print line

#  Build driver

gprbuild('check_mem')

#  Run driver (2 loops)

run('check_mem', ['2'], output_file='check_mem.res1')
run('gnatmem', ['3', '-i', 'gmem.out', './check_mem', "2"],
    output_file='check_mem.run1')

#  Run driver (30 loops)

run('check_mem', ['30'], output_file='check_mem.res2')
run('gnatmem', ['3', '-i', 'gmem.out', './check_mem', "30"],
    output_file='check_mem.run2')

#  Now check that final water-mark for run1 and run2 is equal
r1 = open('check_mem.run1').readlines()
r2 = open('check_mem.run2').readlines()

fr1 = "1"
fr2 = "2"

for item in r1:
    if item[0:8] == "   Final":
        fr1 = item
for item in r2:
    if item[0:8] == "   Final":
        fr2 = item

if fr1 != fr2:
    print "NOK"
    print "run 1 ---------------"
    print_file(r1)
    print "run 2 ---------------"
    print_file(r2)
    sys.exit(1)
else:
    print "OK"
    sys.exit(0)



from test_support import *
import datetime
import time

def check(name, current, expected):
    if current == expected:
        print "OK " + name
    else:
        print "NOK " + name + ": '" + expected + "' != '" + current + "'"

def check_n(name, current, expected, shift=1):
    v = int (current)
    if v == expected or (v > expected - shift and v < expected + shift):
        print "OK " + name
    else:
        print "NOK " + name + ": '" + str(expected) + "' != '" + current + "'"

WeekDays = ["Monday", "Tuesday", "Wednesday", "Thursday", "Friday",
           "Saturday", "Sunday"]

Months = ["January", "February", "March", "April", "May", "June", "July",
          "August", "September", "October", "November", "December"]

now = datetime.datetime.now()

#  Make sure we are not going to change minute in the middle of the test

while now.second > 56:
    time.sleep(5)
    now = datetime.datetime.now()

run('testme', ["testme.tmplt"], output_file="dt.out")
run('print_tree', ["testme.tmplt"])

l1 = WeekDays[now.weekday()] + " " + Months[now.month - 1]
l2 = now.strftime ("%Y/%m/%d")
l3 = now.hour
l4 = now.minute
l5 = now.second

#  Now check dt.out result

file = open ('dt.out')
lines = file.readlines()

check ("L1", lines[0][:-1], l1)
check ("L2", lines[1][:-1], l2)
check_n ("L3", lines[2][:-1], l3)
check_n ("L4", lines[3][:-1], l4)
check_n ("L5", lines[4][:-1], l5, 5)

file.close()



from test_support import *

run('templates2ada', ["-e", "tmpl", "-d", ".", "-t", "./localtempl.tads",
                      "-o", "t2a.tmp"] )
file = open("t2a.tmp")
lines = file.readlines()
file.close()

for line in lines:
    print line[:-1].replace("\\", "/")



from test_support import *

Current = "EndTag"


def set_start(str):
    global Current
    if str.find("SimpleTag") != -1:
        Current = "SimpleTag"
    elif str.find("CompositeTag") != -1:
        Current = "CompositeTag"
    else:
        Current = "EndTag"


def is_end(str):
    global Current
    if Current == "SimpleTag":
        return str.find("</SimpleTag>") != -1
    elif Current == "CompositeTag":
        return str.find("</CompositeTag>") != -1
    else:
        return True


def filter(infile):
    file = open(infile)
    lines = file.readlines()
    file.close()

    txt = []
    #  Copy headers
    txt.append(lines[0][:-1])
    txt.append(lines[1][:-1])

    l = 2
    data = ""

    while l < len(lines):
        v = lines[l][:-1]
        data = data + v.translate(None, ' ')
        set_start(v)
        l = l + 1

        while not is_end(v) and l < len(lines):
            v = lines[l][:-1]
            l = l + 1
            data = data + v.translate(None, ' ')

        txt.append(data)
        data = ""

    return txt.sort()

gprbuild('translations_demo')
run('translations_demo')

res1 = filter("ts.xml")
res2 = filter("tsr.xml")

if res1 == res2:
    print "OK"
else:
    print "NOK"




import os
import os.path

API = ['src/templates_parser.ads',
       'src/templates_parser-debug.ads',
       'src/templates_parser-utils.ads',
       'xsrc/templates_parser-xml.ads']

if os.path.exists("build/apirefs") is False:
    os.makedirs("build/apirefs")

for path in API:
    try:
        fin = open("../" + path)
        fout = open("build/apirefs/" + os.path.basename(path), 'w')
        outside_private_part = True
        for line in fin:
            if line.startswith("end "):
                outside_private_part = True
            if outside_private_part:
                fout.write(line)
            if line == "private\n":
                outside_private_part = False
                fout.write("   -- implementation removed\n")
    finally:
        if fin:
            fin.close()
        if fout:
            fout.close()



"""
This module contains support functions for all test.py
"""

import os
import sys

#  Change directory

TEST = sys.modules['__main__']
TESTDIR = os.path.dirname(TEST.__file__)
TEST_NAME = os.path.basename(TESTDIR)
os.chdir(TESTDIR)

from gnatpython.ex import Run


def gprbuild(prj):
    """Compile a project with gprbuild"""
    cmd = ["gprbuild", "-p", "-gnat05", "-P" + prj, "-bargs", "-E"]
    process = Run(cmd)
    if process.status:
        print process.out


def run(bin, options=None, output_file=None):
    """Run a test"""
    if options is None:
        options = []
    if "TIMEOUT" in os.environ:
        timeout = int(os.environ["TIMEOUT"])
    else:
        timeout = 300

    Run([bin] + options, output=output_file, timeout=timeout)
